package com.yutou.popdialog

import android.animation.Animator
import android.animation.ObjectAnimator
import android.annotation.SuppressLint
import android.app.Activity
import android.app.Dialog
import android.graphics.Color
import android.view.*
import android.view.animation.Animation
import android.view.animation.AnimationUtils
import android.widget.FrameLayout
import android.widget.LinearLayout
import android.widget.TextView
import androidx.appcompat.view.ContextThemeWrapper
import androidx.core.view.GestureDetectorCompat
import com.yutou.popdialog.utils.POPDialogUtils
import com.yutou.popdialog.views.ExitButton


class POPDialog : Dialog {
    private val activityContext: Activity
    private var defLayoutResId = R.layout.pop_dialog_layout
    private lateinit var title: TextView
    private val layout: ViewGroup by lazy {
        findViewById(R.id.pop_dialog_title)
    }
    private val dialogContext: ViewGroup by lazy {
        findViewById(R.id.pop_dialog_content)
    }

    private lateinit var view: View
    private lateinit var mainView: FrameLayout
    private lateinit var touch: GestureDetectorCompat

    private var touchHook = false
    private var showModel = 0
    private var expansionY = 0f

    constructor(context: Activity) : super(context) {
        activityContext = context
        requestWindowFeature(Window.FEATURE_NO_TITLE)
        initView()
    }

    constructor(context: Activity, layoutResID: Int) : super(context) {
        defLayoutResId = layoutResID
        activityContext = context
        requestWindowFeature(Window.FEATURE_NO_TITLE)
        initView()
    }

    private fun initView() {
        mainView = FrameLayout(activityContext)
        view =
            LayoutInflater.from(context).inflate(defLayoutResId, mainView, false)
        val animation = AnimationUtils.loadAnimation(
            context,
            R.anim.pop_dialog_enter_anim
        )
        view.startAnimation(animation)
        val windowParams = FrameLayout.LayoutParams(
            ViewGroup.LayoutParams.MATCH_PARENT,
            ViewGroup.LayoutParams.WRAP_CONTENT
        )
        windowParams.gravity = Gravity.BOTTOM
        view.layoutParams = windowParams
        view.y=+POPDialogUtils.getDisplay(activityContext).mode.physicalHeight.toFloat()
        mainView.addView(view)
        super.setContentView(mainView)
    }

    /**
     * 设置标题文本
     * @param title 内容
     */
    override fun setTitle(title: CharSequence?) {
        this@POPDialog.title = TextView(ContextThemeWrapper(activityContext, R.style.DialogTitle))
        this@POPDialog.title.text = title
        layout.addView(this@POPDialog.title)
    }

    /**
     * 设置标题View
     * @param view 内容
     */
    fun setTitle(view: View) {
        layout.addView(view)
    }

    /**
     * 设置正文文本
     * @param text 内容
     */
    public fun setMessage(text: String) {
        val textView = TextView(ContextThemeWrapper(activityContext, R.style.messageText))
        val params = ViewGroup.LayoutParams(
            FrameLayout.LayoutParams.MATCH_PARENT,
            FrameLayout.LayoutParams.MATCH_PARENT
        )
        textView.layoutParams = params
        textView.text = text
        dialogContext.addView(textView)
    }

    /**
     * 启用关闭按钮
     * 默认false
     */
    fun enableExitButton(enable: Boolean) {
        if (enable) {
            val exitButton = ExitButton(activityContext)
            val params = FrameLayout.LayoutParams(
                ViewGroup.LayoutParams.WRAP_CONTENT,
                ViewGroup.LayoutParams.WRAP_CONTENT
            )
            params.gravity = Gravity.RIGHT
            exitButton.layoutParams = params
            exitButton.setOnClickListener {
                if (showModel == 1) {
                    expansion(false)
                } else {
                    finish()
                }
            }
            layout.addView(exitButton)
        }
    }

    /**
     * 自定义正文view
     * @param view 正文
     */
    override fun setContentView(view: View) {
        dialogContext.addView(view)
    }

    /**
     * 不允许设置资源id
     * @param layoutResID 正文id
     */
    @Deprecated(
        "设置资源id会导致内容无法控制，不建议使用",
        ReplaceWith("throw RuntimeException")
    )
    override fun setContentView(layoutResID: Int) {
        throw RuntimeException(context.resources.getString(R.string.pop_dialog_exception_disable_content_layoutId))
    }

    /**
     * 配置Dialog参数
     */
    private fun configDialog() {
        val display: Display = POPDialogUtils.getDisplay(activityContext)

        window!!.setLayout(
            WindowManager.LayoutParams.MATCH_PARENT,
            WindowManager.LayoutParams.MATCH_PARENT
        )

        window!!.setWindowAnimations(R.style.dialog_pop_anim)
        val attributes = window?.attributes!!
        attributes.windowAnimations = R.anim.pop_dialog_enter_anim
        attributes.x = 0
        attributes.y = 0
        window!!.attributes = attributes
        window!!.decorView.minimumHeight = display.mode.physicalHeight
        window!!.decorView.minimumWidth = display.mode.physicalWidth
        window!!.decorView.setBackgroundColor(Color.parseColor("#10000000"))
        window!!.decorView.setPadding(0, 0, 0, 0)
    }


    /**
     * 展开对话框
     * @param expansion true 展开 false 拉下
     */
    private fun expansion(expansion: Boolean) {
        if (showModel == 0 && !expansion) {
            finish()
            return
        }
        val viewAnim = if (expansion) {
            ObjectAnimator.ofFloat(view, "translationY", 0f).setDuration(300L)
        } else {
            ObjectAnimator.ofFloat(view, "translationY", expansionY).setDuration(300L)
        }

        viewAnim.addListener(object : Animator.AnimatorListener {
            override fun onAnimationStart(p0: Animator?) {
            }

            override fun onAnimationEnd(p0: Animator?) {
                showModel = if (expansion) {
                    1
                } else {
                    0
                }
            }

            override fun onAnimationCancel(p0: Animator?) {
            }

            override fun onAnimationRepeat(p0: Animator?) {
            }

        })
        viewAnim.start()
    }

    /**
     * 获取正文view所占用大小
     */
    override fun onWindowFocusChanged(hasFocus: Boolean) {
        super.onWindowFocusChanged(hasFocus)
        if (hasFocus) {
            if (dialogContext.height > (POPDialogUtils.getDisplay(activityContext).mode.physicalHeight / 3)) {
                touchHook = true
                val params = LinearLayout.LayoutParams(
                    ViewGroup.LayoutParams.MATCH_PARENT,
                    ViewGroup.LayoutParams.WRAP_CONTENT
                )
                params.height = (dialogContext.height / 1.25).toInt()
                dialogContext.layoutParams = params
                view.y =
                    (POPDialogUtils.getDisplay(activityContext).mode.physicalHeight / 2).toFloat()
                expansionY = view.y
            }
            configTouch()
        }
    }

    /**
     * 配置触摸事件
     */
    @SuppressLint("ClickableViewAccessibility")
    private fun configTouch() {
        val layout = FrameLayout(activityContext)
        val params = FrameLayout.LayoutParams(
            ViewGroup.LayoutParams.WRAP_CONTENT,
            ViewGroup.LayoutParams.MATCH_PARENT
        )
        params.width = (POPDialogUtils.getDisplay(activityContext).mode.physicalWidth * 0.8).toInt()
        params.gravity = Gravity.CENTER
        layout.layoutParams = params
        layout.setOnTouchListener(object : View.OnTouchListener {
            override fun onTouch(p0: View?, p1: MotionEvent?): Boolean {
                if (touchHook) {
                    touch.onTouchEvent(p1)
                    return true
                }
                return false
            }

        })
        this@POPDialog.layout.addView(layout)
    }

    /**
     * 显示弹窗
     */
    override fun show() {
        configDialog()
        super.show()
        touch = GestureDetectorCompat(activityContext, DialogTouch())
    }

    /**
     * 结束弹窗
     */
    fun finish() {
        val animation = AnimationUtils.loadAnimation(
            context,
            R.anim.pop_dialog_exit_anim
        )
        animation.setAnimationListener(object : Animation.AnimationListener {
            override fun onAnimationStart(p0: Animation?) {
            }

            override fun onAnimationEnd(p0: Animation?) {
                dismiss()
            }

            override fun onAnimationRepeat(p0: Animation?) {
            }

        })
        view.startAnimation(animation)
    }

    override fun onTouchEvent(event: MotionEvent): Boolean {
        return touch.onTouchEvent(event)
    }

    /**
     * 触摸事件
     */
    private inner class DialogTouch : GestureDetector.SimpleOnGestureListener() {
        override fun onScroll(
            e1: MotionEvent?,
            e2: MotionEvent?,
            distanceX: Float,
            distanceY: Float
        ): Boolean {
            super.onScroll(e1, e2, distanceX, distanceY)
            if ((e2!!.y - e1!!.y) < -100f) {
                expansion(true)
                return true
            } else if ((e2.y - e1.y) > 100f) {
                expansion(false)
                return true
            }
            return false
        }
    }
}
